import greenfoot.*;
import java.util.HashMap;
import java.util.Map;

/**
 * Handles Sounds. Usage:
 * 
 * Call from other classes
 *     Sound.play("explosion.wav")
 * and
 *     Sound.stop("explosion.wav");
 * or
 *     Sound.register("ACK", "ack.mp3");
 *     Sound.play("ACK");
 *     Sound.stop("ACK");
 * also
 *     Sound.stopAll();
 * also
 *     Sound.initializeDefaultSounds();
 *     Sound.play("SAD")
 */
public enum Sound
{
    ; // entry-less enum to prevent creation of any instances of static state class.

    static boolean soundIsOn = false;
    private static final Map<String, GreenfootSound> sounds = new HashMap<>();

    public static void initializeDefaultSounds() { // not necessary
        register("ACK", "PEETIE_Sound_02.mp3");
        register("NO", "PEETIE_Sound_18-Nein.mp3");

        register("STEP1", "step.wav");
        register("STEP2", "step2.wav"); // gerade aufgenommen, ersetzen
        register("STEP3", "step3.wav"); // gerade aufgenommen, ersetzen
        register("STEP4", "step4.wav"); // gerade aufgenommen, ersetzen
        register("BATTERYEATEN", "batteryEaten.wav");
        register("YEAH!", "PEETIE_Sound_01.mp3");
        register("EXPLOSION", "explosion.wav");
        register("GRABEN", "graben.wav");
        register("ABSTURZ", "absturz.wav");
        register("STEINSCHLAG", "steinschlag.wav"); 
        register("RAKETENEHMEN", "raketenehmen.wav");
        register("ENDLOSSCHLEIFE", "endlosschleife.wav");
        register("BING", "bing.wav");
        register("SHOOT", "shoot.wav");
        register("SMALLHIT", "smallhit.wav");
        register("MISSILE", "missile.wav");
        register("BGMUSIC", "EFM_04_ShortVersion.mp3");
        register("LOADING", "loading.wav");
    }

    public static void register(final String fileName) {
        register(fileName, fileName);
    }

    public static void register(final String soundName, final String fileName) {
        final GreenfootSound alreadyRegisteredSound = sounds.get(soundName);
        if (alreadyRegisteredSound != null) {
            alreadyRegisteredSound.stop(); // make sure that old sound is stopped before replacing it.
        }
        sounds.put(soundName, create(fileName));

    }

    private static GreenfootSound create(final String fileName) {
        return new GreenfootSound(fileName);
    }

    public static void play(final String soundName) {
        final GreenfootSound sound = sounds.get(soundName);
        if(soundIsOn) {
            try {

                if (sound == null) { // register, if not yet existing.
                    register(soundName);
                }
                if (!sound.isPlaying()) {
                    sound.play();
                }
            } catch(Exception e) {
                System.out.println("Not existing sound has been called - not executed");           }
        }
    }

    public static void stop(final String soundName) {
        final GreenfootSound sound = get(soundName);
        if (sound != null) {
            sound.stop();
        }
    }

    public static void stopAll() {
        for(final GreenfootSound sound : sounds.values()) {
            sound.stop();
        }
    }

    private static GreenfootSound get(final String soundName) {
        final GreenfootSound result = sounds.get(soundName);
        if (result == null) {
            System.out.println("ERROR - Sound " + soundName + " was called but doesn't exist. Check class SoundHelper.");
        }
        return result;
    }

    public static void toggleSoundIsOn() {
        soundIsOn = !soundIsOn;
    }

}